=== modified file '.bzrignore'
--- .bzrignore	2008-07-19 02:55:28 +0000
+++ .bzrignore	2008-08-03 22:14:59 +0000
@@ -113,3 +113,4 @@
 support-files/drizzle.server
 support-files/libdrizzle.pc
 tests/install_test_db
+drizzled/sql_builtin.cc

=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2008-08-01 17:09:24 +0000
+++ drizzled/Makefile.am	2008-08-03 22:14:59 +0000
@@ -52,6 +52,7 @@
 drizzledincludedir = ${includedir}/drizzled
 dist_drizzledinclude_HEADERS =	error.h \
 				plugin.h \
+				plugin_authentication.h \
 				plugin_audit.h
 
 noinst_LTLIBRARIES = libhandler.la
@@ -72,6 +73,7 @@
 		 $(LIBDL_LIBS) $(LIBEVENT_LIBS)
 
 noinst_HEADERS = item.h \
+		 authentication.h \
 		 item_func.h \
 		 item_sum.h \
 		 item_cmpfunc.h \
@@ -129,6 +131,7 @@
 		 probes.h
 
 drizzled_SOURCES = sql_lex.cc \
+		   authentication.cc \
 		   sql_handler.cc \
 		   item.cc \
 		   item_sum.cc \

=== added file 'drizzled/authentication.cc'
--- drizzled/authentication.cc	1970-01-01 00:00:00 +0000
+++ drizzled/authentication.cc	2008-08-03 22:14:59 +0000
@@ -0,0 +1,72 @@
+#include <drizzled/authentication.h>
+
+static bool are_plugins_loaded= false;
+
+static bool authenticate_by(THD *thd, plugin_ref plugin, void* p_data)
+{
+  const char *password= (const char *)p_data;
+  authentication_st *auth= plugin_data(plugin, authentication_st *);
+
+  (void)p_data;
+
+  if (auth && auth->authenticate)
+  {
+    if (auth->authenticate(thd, password))
+      return true;
+  }
+
+  return false;
+}
+
+bool authenticate_user(THD *thd, const char *password)
+{
+  /* If we never loaded any auth plugins, just return true */
+  if (are_plugins_loaded != true)
+    return true;
+
+  printf("PASSWORD :%s:\n", password);
+
+  return plugin_foreach(thd, authenticate_by, MYSQL_AUTH_PLUGIN, (void *)password);
+}
+
+
+int authentication_initializer(st_plugin_int *plugin)
+{
+  authentication_st *authen;
+
+  if ((authen= (authentication_st *)malloc(sizeof(authentication_st))) == 0)
+      return(1);
+
+  memset(authen, 0, sizeof(authentication_st));
+
+  if (plugin->plugin->init)
+  {
+    if (plugin->plugin->init(authen))
+    {
+      sql_print_error("Plugin '%s' init function returned error.",
+                      plugin->name.str);
+      goto err;
+    }
+  }
+
+  plugin->data= (void *)authen;
+  are_plugins_loaded= true;
+
+  return(0);
+err:
+  free(authen);
+  return(1);
+}
+
+int authentication_finalizer(st_plugin_int *plugin)
+{
+  authentication_st *authen= (authentication_st *)plugin->data;
+
+  assert(authen);
+  if (authen && plugin->plugin->deinit)
+    plugin->plugin->deinit(authen);
+
+  free(authen);
+
+  return(0);
+}

=== added file 'drizzled/authentication.h'
--- drizzled/authentication.h	1970-01-01 00:00:00 +0000
+++ drizzled/authentication.h	2008-08-03 22:14:59 +0000
@@ -0,0 +1,13 @@
+#ifndef DRIZZLED_AUTHENTICATION_H
+#define DRIZZLED_AUTHENTICATION_H
+
+#include <mysql_priv.h>
+#include <drizzled/sql_plugin.h>
+#include <drizzled/plugin.h>
+#include <drizzled/plugin_authentication.h>
+
+bool authenticate_user(THD *thd, const char *password);
+int authentication_initializer(st_plugin_int *plugin);
+int authentication_finalizer(st_plugin_int *plugin);
+
+#endif /* DRIZZLED_AUTHENTICATION_H */

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2008-08-01 22:38:22 +0000
+++ drizzled/drizzled.cc	2008-08-03 22:14:59 +0000
@@ -198,7 +198,7 @@
 /*
   Used with --help for detailed option
 */
-static bool opt_help= 0, opt_verbose= 0;
+static bool opt_help= false;
 
 arg_cmp_func Arg_comparator::comparator_matrix[5][2] =
 {{&Arg_comparator::compare_string,     &Arg_comparator::compare_e_string},
@@ -3455,9 +3455,6 @@
    IF_PURIFY(0,1), 0, 0, 0, 0, 0},
   {"user", 'u', "Run mysqld daemon as user.", 0, 0, 0, GET_STR, REQUIRED_ARG,
    0, 0, 0, 0, 0, 0},
-  {"verbose", 'v', "Used with --help option for detailed help",
-   (char**) &opt_verbose, (char**) &opt_verbose, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0,
-   0, 0},
   {"version", 'V', "Output version information and exit.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"warnings", 'W', "Deprecated; use --log-warnings instead.",
@@ -4111,23 +4108,22 @@
 Copyright (C) 2000 MySQL AB, by Monty and others\n\
 This software comes with ABSOLUTELY NO WARRANTY. This is free software,\n\
 and you are welcome to modify and redistribute it under the GPL license\n\n\
-Starts the MySQL database server\n");
+Starts the Drizzle database server\n");
 
   printf("Usage: %s [OPTIONS]\n", my_progname);
-  if (!opt_verbose)
-    puts("\nFor more help options (several pages), use mysqld --verbose --help");
-  else
   {
+#ifdef FOO
   print_defaults(MYSQL_CONFIG_NAME,load_default_groups);
   puts("");
   set_ports();
+#endif
 
   /* Print out all the options including plugin supplied options */
   my_print_help_inc_plugins(my_long_options, sizeof(my_long_options)/sizeof(my_option));
 
   puts("\n\
 To see what values a running DrizzleD server is using, type\n\
-'drizzleadmin variables' instead of 'mysqld --verbose --help'.");
+'drizzleadmin variables' instead of 'drizzled --help'.");
   }
 }
 

=== added file 'drizzled/plugin_authentication.h'
--- drizzled/plugin_authentication.h	1970-01-01 00:00:00 +0000
+++ drizzled/plugin_authentication.h	2008-08-03 22:14:59 +0000
@@ -0,0 +1,13 @@
+/*
+  Definitions required for Authentication plugin 
+*/
+
+#ifndef DRIZZLED_PLUGIN_AUTHENTICATION_H
+#define DRIZZLED_PLUGIN_AUTHENTICATION_H
+
+typedef struct authentication_st
+{
+  bool (*authenticate)(THD *thd, const char *password);
+} authentication_st;
+
+#endif /* DRIZZLED_PLUGIN_AUTHENTICATION_H */

=== modified file 'drizzled/sql_connect.cc'
--- drizzled/sql_connect.cc	2008-07-29 07:47:38 +0000
+++ drizzled/sql_connect.cc	2008-08-03 22:14:59 +0000
@@ -19,6 +19,7 @@
 */
 
 #include "mysql_priv.h"
+#include <drizzled/authentication.h>
 
 #define MIN_HANDSHAKE_SIZE      6
 
@@ -77,11 +78,12 @@
 
 int
 check_user(THD *thd, enum enum_server_command command,
-           const char *passwd __attribute__((unused)),
+           const char *passwd,
            uint passwd_len, const char *db,
            bool check_count)
 {
   LEX_STRING db_str= { (char *) db, db ? strlen(db) : 0 };
+  bool is_authenticated;
 
   /*
     Clear thd->db as it points to something, that will be freed when
@@ -91,29 +93,25 @@
   */
   thd->reset_db(NULL, 0);
 
-  bool opt_secure_auth_local;
-  pthread_mutex_lock(&LOCK_global_system_variables);
-  opt_secure_auth_local= opt_secure_auth;
-  pthread_mutex_unlock(&LOCK_global_system_variables);
-
-  /*
-    If the server is running in secure auth mode, short scrambles are 
-    forbidden.
-  */
-  if (opt_secure_auth_local && passwd_len == SCRAMBLE_LENGTH_323)
-  {
-    my_error(ER_NOT_SUPPORTED_AUTH_MODE, MYF(0));
-    general_log_print(thd, COM_CONNECT, ER(ER_NOT_SUPPORTED_AUTH_MODE));
-    return(1);
-  }
-  if (passwd_len != 0 &&
-      passwd_len != SCRAMBLE_LENGTH &&
-      passwd_len != SCRAMBLE_LENGTH_323)
+  if (passwd_len != 0 && passwd_len != SCRAMBLE_LENGTH)
   {
     my_error(ER_HANDSHAKE_ERROR, MYF(0), thd->main_security_ctx.host_or_ip);
     return(1);
   }
 
+  is_authenticated= authenticate_user(thd, passwd);
+
+  if (is_authenticated != true)
+  {
+    my_error(ER_ACCESS_DENIED_ERROR, MYF(0),
+             thd->main_security_ctx.user,
+             thd->main_security_ctx.host_or_ip,
+             passwd_len ? ER(ER_YES) : ER(ER_NO));
+
+    return 1;
+  }
+
+
   USER_RESOURCES ur;
   thd->security_ctx->skip_grants();
   memset(&ur, 0, sizeof(USER_RESOURCES));
@@ -327,9 +325,6 @@
       return 1;
     }
   }
-#ifdef _CUSTOMCONFIG_
-#include "_cust_sql_parse.h"
-#endif
   if (thd->packet.alloc(thd->variables.net_buffer_length))
     return 1; /* The error is set by alloc(). */
 

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2008-08-01 22:38:22 +0000
+++ drizzled/sql_parse.cc	2008-08-03 22:14:59 +0000
@@ -567,6 +567,7 @@
     /* Safe because there is always a trailing \0 at the end of the packet */
     char *passwd= strend(user)+1;
 
+
     thd->clear_error();                         // if errors from rollback
 
     /*

=== modified file 'drizzled/sql_plugin.cc'
--- drizzled/sql_plugin.cc	2008-07-29 07:47:38 +0000
+++ drizzled/sql_plugin.cc	2008-08-03 22:14:59 +0000
@@ -16,6 +16,7 @@
 #include "mysql_priv.h"
 #include <mysys/my_pthread.h>
 #include <mysys/my_getopt.h>
+#include <authentication.h>
 #define REPORT_TO_LOG  1
 #define REPORT_TO_USER 2
 
@@ -63,7 +64,7 @@
   0,  /* UDA */
   0,  /* Audit */
   0,  /* Logger */
-  0  /* Auth */
+  authentication_initializer  /* Auth */
 };
 
 plugin_type_init plugin_type_deinitialize[MYSQL_MAX_PLUGIN_TYPE_NUM]=
@@ -75,7 +76,7 @@
   0,  /* UDA */
   0,  /* Audit */
   0,  /* Logger */
-  0  /* Auth */
+  authentication_finalizer  /* Auth */
 };
 
 static const char *plugin_declarations_sym= "_mysql_plugin_declarations_";

=== modified file 'drizzled/sql_show.cc'
--- drizzled/sql_show.cc	2008-08-01 20:40:12 +0000
+++ drizzled/sql_show.cc	2008-08-03 22:14:59 +0000
@@ -2153,7 +2153,7 @@
   add_data.wild= wild;
   if (plugin_foreach(thd, add_schema_table,
                      MYSQL_INFORMATION_SCHEMA_PLUGIN, &add_data))
-      return(1);
+    return(1);
 
   return(0);
 }

=== modified file 'plugin/Makefile.am'
--- plugin/Makefile.am	2008-07-13 08:00:16 +0000
+++ plugin/Makefile.am	2008-08-03 22:14:59 +0000
@@ -19,7 +19,6 @@
 
 # extra plugin example files are listed here, to
 # keep its Makefile.am cleaner as a template
-#EXTRA_DIST =	fulltext/configure.in	
 
 SUBDIRS =	@mysql_pg_dirs@
-DIST_SUBDIRS =	hello_world md5 crc32
+DIST_SUBDIRS =	auth_pam hello_world md5 crc32

=== added directory 'plugin/auth_pam'
=== added file 'plugin/auth_pam/AUTHORS'
--- plugin/auth_pam/AUTHORS	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/AUTHORS	2008-08-03 22:14:59 +0000
@@ -0,0 +1,2 @@
+Brian Aker <brian@tangent.org> -- Initial Version
+

=== added file 'plugin/auth_pam/ChangeLog'
--- plugin/auth_pam/ChangeLog	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/ChangeLog	2008-08-03 22:14:59 +0000
@@ -0,0 +1,2 @@
+0.1
+  - Added

=== added file 'plugin/auth_pam/Makefile.am'
--- plugin/auth_pam/Makefile.am	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/Makefile.am	2008-08-03 22:14:59 +0000
@@ -0,0 +1,27 @@
+# Copyright (C) 2006 MySQL AB
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+MYSQLDATAdir =          $(localstatedir)
+MYSQLSHAREdir =         $(pkgdatadir)
+MYSQLBASEdir=           $(prefix)
+MYSQLLIBdir=            $(pkglibdir)
+pkgplugindir =		$(pkglibdir)/plugin
+
+AM_CPPFLAGS = 		${GLOBAL_CPPFLAGS} -I${top_srcdir}/drizzled
+EXTRA_LTLIBRARIES =	libauth_pam.la
+pkgplugin_LTLIBRARIES =	@plugin_auth_pam_shared_target@
+libauth_pam_la_LDFLAGS =	-module -rpath $(pkgplugindir) -lpam -lpam_misc
+libauth_pam_la_CPPFLAGS=	$(AM_CPPFLAGS) -DMYSQL_DYNAMIC_PLUGIN
+libauth_pam_la_SOURCES =	auth_pam.cc

=== added file 'plugin/auth_pam/README'
--- plugin/auth_pam/README	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/README	2008-08-03 22:14:59 +0000
@@ -0,0 +1,19 @@
+Hi!
+
+First, as experimental goes, this is pretty experimental.
+
+Next thing, I do not really know PAM. What I do know is how to read man
+pages :)
+
+Go place this:
+auth       required     pam_unix.so
+account    required     pam_unix.so
+
+Into a file located here:
+/etc/pam.d/drizzled
+
+Please send email to the drizzle mailing list on Launchpad if you would like
+to contribute.
+
+Cheers,
+    -Brian

=== added file 'plugin/auth_pam/auth_pam.cc'
--- plugin/auth_pam/auth_pam.cc	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/auth_pam.cc	2008-08-03 22:14:59 +0000
@@ -0,0 +1,121 @@
+/*
+  Sections of this where taken/modified from mod_auth_path for Apache 
+*/
+
+#define MYSQL_SERVER 1
+#include <drizzled/mysql_priv.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <drizzled/plugin.h>
+#include <drizzled/plugin_authentication.h>
+#include <security/pam_appl.h>
+#include <security/pam_misc.h>
+
+
+typedef struct {
+    const char *name;
+    const char *password;
+} auth_pam_userinfo;
+
+static int auth_pam_talker(int num_msg,
+                           const struct pam_message **msg,
+                           struct pam_response **resp,
+                           void *appdata_ptr)
+{
+  auth_pam_userinfo *userinfo = (auth_pam_userinfo*)appdata_ptr;
+  struct pam_response *response = 0;
+  int x;
+
+  /* parameter sanity checking */
+  if(!resp || !msg || !userinfo)
+    return PAM_CONV_ERR;
+
+  /* allocate memory to store response */
+  response= (struct pam_response*)malloc(num_msg * sizeof(struct pam_response));
+  if(!response)
+    return PAM_CONV_ERR;
+
+  /* copy values */
+  for(x= 0; x < num_msg; x++) 
+  {
+    /* initialize to safe values */
+    response[x].resp_retcode= 0;
+    response[x].resp= 0;
+
+    /* select response based on requested output style */
+    switch(msg[x]->msg_style) 
+    {
+    case PAM_PROMPT_ECHO_ON:
+      /* on memory allocation failure, auth fails */
+      response[x].resp = strdup(userinfo->name);
+      break;
+    case PAM_PROMPT_ECHO_OFF:
+      response[x].resp = strdup(userinfo->password);
+      break;
+    default:
+      if(response)
+        free(response);
+      return PAM_CONV_ERR;
+    }
+  }
+
+  /* everything okay, set PAM response values */
+  *resp = response;
+
+  return PAM_SUCCESS;
+}
+
+static bool authenticate(THD *thd, const char *password)
+{
+  int retval;
+  auth_pam_userinfo userinfo= { NULL, NULL };
+  struct pam_conv conv_info= { &auth_pam_talker, (void*)&userinfo };
+  pam_handle_t *pamh= NULL;
+
+  userinfo.name= thd->main_security_ctx.user;
+  userinfo.password= password;
+
+  retval= pam_start("check_user", userinfo.name, &conv_info, &pamh);
+
+  if (retval == PAM_SUCCESS)
+    retval= pam_authenticate(pamh, PAM_DISALLOW_NULL_AUTHTOK);
+
+  if (retval == PAM_SUCCESS)
+    retval= pam_acct_mgmt(pamh, PAM_DISALLOW_NULL_AUTHTOK);
+
+  pam_end(pamh, retval);
+
+  return (retval == PAM_SUCCESS) ? true: false;
+}
+
+static int initialize(void *p)
+{
+  authentication_st *auth= (authentication_st *)p;
+  
+  auth->authenticate= authenticate;
+
+  return 0;
+}
+
+static int finalize(void *p)
+{
+  (void)p;
+
+  return 0;
+}
+
+mysql_declare_plugin(auth_pam)
+{
+  MYSQL_AUTH_PLUGIN,
+  "pam",
+  "0.1",
+  "Brian Aker",
+  "PAM based authenication.",
+  PLUGIN_LICENSE_GPL,
+  initialize, /* Plugin Init */
+  finalize, /* Plugin Deinit */
+  NULL,   /* status variables */
+  NULL,   /* system variables */
+  NULL    /* config options */
+}
+mysql_declare_plugin_end;

=== added file 'plugin/auth_pam/configure.in'
--- plugin/auth_pam/configure.in	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/configure.in	2008-08-03 22:14:59 +0000
@@ -0,0 +1,7 @@
+AC_INIT(auth_pam, 0.1)
+AM_INIT_AUTOMAKE
+AC_DISABLE_STATIC
+AC_PROG_LIBTOOL
+AC_CONFIG_FILES([Makefile])
+AC_OUTPUT
+

=== added file 'plugin/auth_pam/plug.in'
--- plugin/auth_pam/plug.in	1970-01-01 00:00:00 +0000
+++ plugin/auth_pam/plug.in	2008-08-03 22:14:59 +0000
@@ -0,0 +1,3 @@
+MYSQL_PLUGIN(auth_pam,[PAM Authenication Plugin],
+        [PAM based authenication.])
+MYSQL_PLUGIN_DYNAMIC(auth_pam,   [libauth_pam.la])

